-module(outil@arg).
-compile([no_auto_import, nowarn_unused_vars]).

-export([bool/3, string/3, float/3, int/3]).

-spec append_argument(outil:command(), outil:argument()) -> outil:command().
append_argument(Cmd, Arg) ->
    {command,
        erlang:element(2, Cmd),
        erlang:element(3, Cmd),
        gleam@list:append(erlang:element(4, Cmd), [Arg]),
        erlang:element(5, Cmd),
        erlang:element(6, Cmd)}.

-spec positional_arg_parser(
    integer(),
    binary(),
    fun((binary()) -> {ok, FDC} | {error, nil})
) -> fun((list(binary())) -> {ok, FDC} | {error, outil@error:reason()}).
positional_arg_parser(At, Name, Parser) ->
    fun(Args) -> case gleam@list:at(Args, At) of
            {error, _} ->
                {error, {missing_argument, Name}};

            {ok, Arg} ->
                case Arg of
                    <<"-"/utf8, _/binary>> ->
                        {error, {out_of_place_option, Arg}};

                    _ ->
                        _pipe = Parser(Arg),
                        gleam@result:map_error(
                            _pipe,
                            fun(_) -> {malformed_argument, Name, Arg} end
                        )
                end
        end end.

-spec with_positional_argument(
    outil:command(),
    outil:argument(),
    fun((binary()) -> {ok, FCX} | {error, nil}),
    fun((fun((outil:command(), fun((FCX) -> {ok, FEP} |
        {error, outil:command_return(FEQ)})) -> {ok, FEP} |
        {error, outil:command_return(FEQ)}), outil:command()) -> FEO)
) -> FEO.
with_positional_argument(Cmd, Argument, Parse, Continue) ->
    Arg_pos = gleam@list:length(erlang:element(4, Cmd)),
    Arg_parser = positional_arg_parser(
        Arg_pos,
        erlang:element(2, Argument),
        Parse
    ),
    Arg_parser@1 = fun(Run_cmd, And_then) ->
        _pipe = outil@help:wrap_usage(Run_cmd, Arg_parser),
        gleam@result:then(_pipe, And_then)
    end,
    Continue(Arg_parser@1, append_argument(Cmd, Argument)).

-spec bool(
    outil:command(),
    binary(),
    fun((fun((outil:command(), fun((boolean()) -> {ok, FES} |
        {error, outil:command_return(FET)})) -> {ok, FES} |
        {error, outil:command_return(FET)}), outil:command()) -> FEU)
) -> FEU.
bool(Cmd, Name, Continue) ->
    with_positional_argument(
        Cmd,
        {bool_argument, Name},
        fun outil:parse_bool/1,
        Continue
    ).

-spec string(
    outil:command(),
    binary(),
    fun((fun((outil:command(), fun((binary()) -> {ok, FEW} |
        {error, outil:command_return(FEX)})) -> {ok, FEW} |
        {error, outil:command_return(FEX)}), outil:command()) -> FEY)
) -> FEY.
string(Cmd, Name, Continue) ->
    with_positional_argument(
        Cmd,
        {string_argument, Name},
        fun(Field@0) -> {ok, Field@0} end,
        Continue
    ).

-spec float(
    outil:command(),
    binary(),
    fun((fun((outil:command(), fun((float()) -> {ok, FFC} |
        {error, outil:command_return(FFD)})) -> {ok, FFC} |
        {error, outil:command_return(FFD)}), outil:command()) -> FFE)
) -> FFE.
float(Cmd, Name, Continue) ->
    with_positional_argument(
        Cmd,
        {float_argument, Name},
        fun gleam@float:parse/1,
        Continue
    ).

-spec int(
    outil:command(),
    binary(),
    fun((fun((outil:command(), fun((integer()) -> {ok, FFG} |
        {error, outil:command_return(FFH)})) -> {ok, FFG} |
        {error, outil:command_return(FFH)}), outil:command()) -> FFI)
) -> FFI.
int(Cmd, Name, Continue) ->
    with_positional_argument(
        Cmd,
        {int_argument, Name},
        fun gleam@int:parse/1,
        Continue
    ).
